﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace CustomSorter
{
    class ReflectionHelper
    {
        public static string GetValue<T>(T parentObj, string compositeFieldName)
        {
            var properties = compositeFieldName.Split(new[] { '.' }, StringSplitOptions.RemoveEmptyEntries);

            object parent = parentObj;

            for (int i = 0; i < properties.Count(); i++)
            {
                var fieldName = properties[i];
                var type = parent.GetType();
               
                var index = fieldName.IndexOf("get_Item");
                if (index >= 0)
                {
                    //with VS2010, you can just do this:  parent = parent[fieldName]

                    var methodInfo = type.GetMethod("get_Item",
                                   BindingFlags.IgnoreCase | BindingFlags.NonPublic | BindingFlags.Public |
                                   BindingFlags.Instance | BindingFlags.Static);

                    fieldName = fieldName.Substring(9, 1);

                    parent = methodInfo.Invoke(parent, new object[] { int.Parse(fieldName) });
                }
                else
                {
                    var fieldInfo = type.GetField(fieldName,
                                                  BindingFlags.IgnoreCase | BindingFlags.NonPublic | BindingFlags.Public |
                                                  BindingFlags.Instance | BindingFlags.Static);
                    if (fieldInfo != null)
                    {
                        parent = fieldInfo.GetValue(parent);
                    }
                    else
                    {
                        var propertyInfo = type.GetProperty(fieldName,
                                                            BindingFlags.IgnoreCase | BindingFlags.NonPublic |
                                                            BindingFlags.Public | BindingFlags.Instance |
                                                            BindingFlags.Static);

                        if (propertyInfo != null) parent = propertyInfo.GetValue(parent, new object[] {});
                    }
                }
            }

            return parent.ToString();
        }


        public static string GetMemberInfo(Expression member)
        {
            if (member == null)
            {
                throw new ArgumentNullException("member");
            }

            var lambda = member as LambdaExpression;

            if (lambda == null)
            {
                throw new ArgumentException("Not a lambda expression", "member");
            }

            var body = lambda.Body.ToString();

            var firstDotIndex = body.IndexOf(".") + 1;

            return body.Substring(firstDotIndex, body.Length - firstDotIndex);
        }
    }
}


