﻿using System;
using System.Web.UI.WebControls;
using System.Collections;
using GenericGridViewHelper;

namespace GenericGridViewDemo
{
    public partial class GenericGridView : System.Web.UI.UserControl
    {
        public IEnumerable DataSource;
        public bool AllowPaging;
        public bool AllowSorting;
        public event GridViewRowEventHandler RowDataBound;
        public event Func<string, string> HeaderFormatting;
        public event Func<GridViewSortEventArgs, IEnumerable> Sorting;

        // at the moment we don't give users the option to change these styles. 
        // Should the needs arises, it is very easy to make row styles customiseable. 
        private const string ROW_STYLE = "row";
        private const string ALTERNATIVE_ROW_STYLE = "altrow";
        private const string SORT_UP_CSS_CLASS = "sortUp";
        private const string SORT_DOWN_CSS_CLASS = "sortDown";

        private int _totalRowCount;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (DataSource == null) return;
            gvGeneric.AllowPaging = AllowSorting;
            gvGeneric.AllowSorting = AllowSorting;
            DataSourceHelper.PrepareDataSource(gvGeneric, ViewState, ref DataSource, ref _totalRowCount, Sorting);
            ColumnHelper.PrepareColumns(DataSource, gvGeneric.Columns, HeaderFormatting);
            NavigationHelper.PreparePaging(gvGeneric, _totalRowCount, ddlRowsPerPage, this, AllowSorting, navigationBar);
        }

        protected void Page_PreRender(object sender, EventArgs e)
        {
            if (!_hasDataBeenBound) gvGeneric.DataBind();
        }

        private bool _hasDataBeenBound;
        protected void GridView_DataBound(Object sender, EventArgs e)
        {
            NavigationHelper.SetUpNavigationBar(gvGeneric, _totalRowCount, imgFirst,
            imgPrevious, imgNext, imgLast, lblStartIndex, lblEndIndex, lblTotalCount);
            StyleHelper.SetHeaderStyle(ViewState, gvGeneric.Columns, SORT_UP_CSS_CLASS, SORT_DOWN_CSS_CLASS);
            _hasDataBeenBound = true;
        }

        protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            StyleHelper.SetRowStyle(e, ROW_STYLE, ALTERNATIVE_ROW_STYLE);
            // give user the chance to customise each cell
            if (RowDataBound != null) RowDataBound(sender, e);
        }

        protected void GridView_Sorting(object sender, GridViewSortEventArgs e)
        {
            // GridViewSortEventArgs.SortDirection won't get populated correctly when the GridView datasource is
            // a list of objects, so we have to manually correct it. 
            SortingHelper.PrepareGridViewSortEventArgs(e, ViewState);
            var dataSource = Sorting != null ? Sorting(e) : null;
            DataSourceHelper.SelectDataSourceAndBind(gvGeneric, dataSource, DataSource, e);
        }

        protected void GridView_Navigate(Object sender, EventArgs e)
        {
            NavigationHelper.PreparePageIndex(gvGeneric, ((ImageButton)sender).ID);
            gvGeneric.DataBind();
        }

        protected void NumberOfItemsPerPage_Changed(Object sender, EventArgs e)
        {
            gvGeneric.PageIndex = 0;
            gvGeneric.DataBind();
        }
    }
}