﻿using System.Collections;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using Xunit.Extensions;

namespace GenericGridViewHelper.Specifications.ColumnHelperSpec
{
    public class When_prepares_columns_with_no_header_formatter : Specification
    {
        private List<object> _dataSource = new List<object>();
        private DataControlFieldCollection _columns = new DataControlFieldCollection();

        public override void Observe()
        {
            _dataSource.Add(new { Name = "Frank", AgeGroup =2 });
      
            ColumnHelper.PrepareColumns(_dataSource, _columns, null);
        }

        [Observation]
        public void should_add_all_properties_as_column_using_default_header_formatter()
        {
            _columns.Count.ShouldEqual(2);
            _columns[0].HeaderText.ShouldEqual("Name");
            _columns[1].HeaderText.ShouldEqual("Age Group"); //default header formatter just separates words
        }
    }


    public class When_prepares_columns_with_header_formatter : Specification
    {
        private List<object> _dataSource = new List<object>();
        private DataControlFieldCollection _columns = new DataControlFieldCollection();

        public override void Observe()
        {
            _dataSource.Add(new { Name = "Frank", AgeGroup = 2 });

            ColumnHelper.PrepareColumns(_dataSource, _columns, formatter);
        }

        private string formatter(string header)
        {
            if (header == "Name")
            {
                return "Big Name";
            }
            else
            {
                return header;
            }
        }

        [Observation]
        public void should_add_all_properties_as_column_using_supplied_header_formatter()
        {
            _columns.Count.ShouldEqual(2);
            _columns[0].HeaderText.ShouldEqual(formatter(_columns[0].HeaderText));
            _columns[1].HeaderText.ShouldEqual(formatter(_columns[1].HeaderText));
        }
    }
}
