﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using Rhino.Mocks;
using Xunit.Extensions;

namespace GenericGridViewHelper.Specifications.NavigationHelperSpecs
{
    public class When_allow_paging_is_true_and_dropdown_list_has_all_selected : Specification
    {
        private GridView _gridView;
        private const int TotalRowCount = 30;
        public override void Observe()
        {
            _gridView = MockRepository.GenerateStub<GridView>();
            var dropdownList = MockRepository.GenerateStub<ListControl>();
            dropdownList.SelectedValue = "all";

            var page = MockRepository.GenerateStub<TemplateControl>();
            var navigation = MockRepository.GenerateStub<Control>();

            NavigationHelper.PreparePaging(_gridView, TotalRowCount, dropdownList, page, true, navigation);
        }

        [Observation]
        public void should_set_gridview_page_size_to_total_row_count()
        {
            _gridView.PageSize.Equals(TotalRowCount);
        }
    }

    public class When_allow_paging_is_false : Specification
    {
        private TemplateControl _page;
        private Control _navigation;

        public override void Observe()
        {
            var gridView = MockRepository.GenerateStub<GridView>();
            var dropdownList = MockRepository.GenerateStub<ListControl>();

            _page = MockRepository.GenerateStub<TemplateControl>();
            _page.Stub(x => x.Controls).Return(new ControlCollection(_page));
          
            _navigation = MockRepository.GenerateStub<Control>();

            _page.Controls.Add(_navigation);

            NavigationHelper.PreparePaging(gridView, 30, dropdownList, _page, false, _navigation);
        }

        [Observation]
        public void should_remove_the_navigation_control_from_the_page()
        {
            _page.Controls.Contains(_navigation).ShouldBeFalse();
        }
    }

}
