﻿
using System.Web.UI.WebControls;
using Xunit.Extensions;

namespace GenericGridViewHelper.Specifications.PagingHelperSpec
{
    public class When_the_user_navigated_to_the_next_page : Specification
    {
        private GridView _gridView;
        private int _currentPageIndex;

        public override void Observe()
        {
            _gridView = new GridView();
            _gridView.PageIndex = 10; // so we can verify that page index is set to 0 

            _currentPageIndex = _gridView.PageIndex;

            NavigationHelper.PreparePageIndex(_gridView, imgButtonID: "imgNext");
        }

        [Observation]
        public void should_advance_page_index_by_1()
        {
            _gridView.PageIndex.ShouldEqual(_currentPageIndex + 1);
        }
    }

    public class When_the_user_navigated_to_the_previous_page : Specification
    {
        private GridView _gridView;
        private int _currentPageIndex;

        public override void Observe()
        {
            _gridView = new GridView();
            _gridView.PageIndex = 10; // so we can verify that page index is set to 0 

            _currentPageIndex = _gridView.PageIndex;

            NavigationHelper.PreparePageIndex(_gridView, imgButtonID: "imgPrevious");
        }

        [Observation]
        public void should_advance_page_index_by_1()
        {
            _gridView.PageIndex.ShouldEqual(_currentPageIndex - 1);
        }
    }

    public class When_the_user_navigated_to_the_first_page : Specification
    {
        private GridView _gridView;

        public override void Observe()
        {
            _gridView = new GridView();
            _gridView.PageIndex = 10; // so we can verify that page index is set to 0 

            NavigationHelper.PreparePageIndex(_gridView, imgButtonID: "imgFirst");
        }

        [Observation]
        public void should_set_page_index_to_0()
        {
            _gridView.PageIndex.ShouldEqual(0);
        }
    }

    public class When_the_user_navigated_to_the_last_page : Specification
    {
        private GridView _gridView;

        public override void Observe()
        {
            _gridView = new GridView();
            _gridView.PageIndex = 10; // so we can verify that page index is set to 0 

            NavigationHelper.PreparePageIndex(_gridView, imgButtonID: "imgLast");
        }

        [Observation]
        public void should_set_page_index_to_the_last_page()
        {
            _gridView.PageIndex.ShouldEqual(_gridView.PageCount);
        }
    }

}
