﻿using System.Web.UI;
using System.Web.UI.WebControls;
using Xunit;
using Xunit.Extensions;

namespace GenericGridViewHelper.Specifications.StyleHelperSpec
{
    public class When_sort_the_grid_by_a_column_by_ascedning_order : Specification
    {
        private string _sortColumn = "columnA";
        private string _ascendingClassName = "_ascendingClassName";
        private string _descendingClassName = "_descendingClassName";

        private BoundField _sortField;
        private BoundField _nonSortField;

        public override void Observe()
        {
            _sortField = new BoundField { DataField = _sortColumn };
            _nonSortField = new BoundField { DataField = "b" };
            _nonSortField.HeaderStyle.CssClass = "non empty";

            var fields = new DataControlFieldCollection { _sortField, _nonSortField };
            var viewState = new StateBag();

            viewState[Const.SortDirection] = SortDirection.Ascending;
            viewState[Const.SortExpression] = _sortColumn;

            StyleHelper.SetHeaderStyle(viewState, fields, _ascendingClassName, _descendingClassName);
        }

        [Observation]
        public void should_format_the_sorting_colomn_header_with_ascending_style()
        {
            _sortField.HeaderStyle.CssClass.ShouldEqual(_ascendingClassName);
            _nonSortField.HeaderStyle.CssClass.ShouldEqual(string.Empty);
        }
    }

    public class When_sort_the_grid_by_a_column_by_descedning_order : Specification
    {
        private string _sortColumn = "columnA";
        private string _ascendingClassName = "_ascendingClassName";
        private string _descendingClassName = "_descendingClassName";

        private BoundField _sortField;
        private BoundField _nonSortField;

        public override void Observe()
        {
            _sortField = new BoundField { DataField = _sortColumn };
            _nonSortField = new BoundField { DataField = "b" };
            _nonSortField.HeaderStyle.CssClass = "non empty";

            var fields = new DataControlFieldCollection { _sortField, _nonSortField };
            var viewState = new StateBag();

            viewState[Const.SortDirection] = SortDirection.Descending;
            viewState[Const.SortExpression] = _sortColumn;

            StyleHelper.SetHeaderStyle(viewState, fields, _ascendingClassName, _descendingClassName);
        }

        [Observation]
        public void should_format_the_sorting_colomn_header_with_descedning_style()
        {
            _sortField.HeaderStyle.CssClass.ShouldEqual(_descendingClassName);
            _nonSortField.HeaderStyle.CssClass.ShouldEqual(string.Empty);
        }
    }
}
